texrpnmath.lua: LaTeX の数式を RPN (的表現) で記述するためのモジュール
======================================================================

Description
-----------

LaTeX の数式を RPN (的表現) で記述するためのモジュールです。
完全な RPN ではありません。


Status
------

Version 0.4

プレベータリリース。
今後、仕様が大幅に変更になる場合があります。


要件
----

* lua: 5.1 or later	(5.3.4 でのみ確認)
* LuaLaTeX			(コードを .tex に埋め込む場合のみ)
* Emacs				(texrpnmath.el を使用する場合, 25.3.1 でのみ確認)


Quick Start
-----------

texrpnmath.lua と sample.tex がある状態で、

> $ lualatex sample

を実行してみて下さい。

texrpnmath.lua 単体でも実行できます。
この場合 `.quit` (または `.exit`) で終了できます。

> $ lua texrpnmath.lua


TeX file への埋め込みの例
----------------------

    \documentclass[a4paper]{article}
    \usepackage{luacode}
    \RequireLuaModule{texrpnmath}
    
    \begin{document}
    \[ \directlua{tex.print(texrpnmath.rpn("a sin b cos frac"))} \]
    \end{document}

このコードの数式部分は、次のコードと等価です:

	\[ \frac{\sin a}{\cos b} \]


Emacs での使用
--------------

インストール手順:

1. `texrpnmath.lua` を適切な場所にコピー
2. `texrpnmath.el` を適切な場所にコピー
3. 以下を `.emacs` に追加 (`SOMEWHERE` は適切に変更して下さい)

		(require 'texrpnmath)
		;; or
		;;	(autoload 'texrpnmath-insert "texrpnmath.el" nil t)
        ;;  (autoload 'texrpnmath-mode "texrpnmath.el" nil t)
		(custom-set-variables '(texrpnmath-script "SOMEWHERE/texrpnmath.lua"))

	その後 `eval` (あるいは Emacs 再起動) し、`M-x texrpnmath-mode` でマイナーモードへ以降します。
	`` ` `` (backquote) で入力開始及び終了です。
	`` `[SPC]a[SPC]sin[SPC]` `` などのように入力します。

4. (参考) AUCTeX で使用する場合の例: 以下を `.emacs` を追加すると
`C-c m`, `C-c ~` に `texrpnmath-insert`, `texrpnmath-mode`
がそれぞれバインドされます。
なお、`C-c ~` は AUCTeX では `LaTeX-math-mode` に割り当てられています。
これをオーバーライドしたくない場合は別のキーを使用するか、 `M-x texrpnmath-mode` で起動して下さい。

        ;; AUCTeX
        (eval-after-load "latex"
          '(progn
             (add-hook 'LaTeX-mode-hook
        	       '(lambda ()
					  (local-set-key "\C-cm" 'texrpnmath-insert)
					  ;; LaTeX-math-mode` をオーバーライド
					  (local-set-key "\C-c~" 'texrpnmath-mode)))))


オペレータ
----------

以下では次の表記を用います:

	a /sin/ "\sin a"

これは「スタック (の先頭) に `a` がある状態で `sin` オペレータを適用すると
`\sin a` がスタックに入る」を意味します。
"" があるときはスペースも含めて一つのスタック要素 (引用符は付かない)
であることを意味します
`/op/` は該当するオペレータ一般をさします。

- 数学関数: `sin`, `cos` など: `a /op/ "\op a"`

  () は付きません

- 引数なしコマンド: `aleph`, `nabla`, `infty` など: `/op/ \op`

- 一引数コマンド: `sqrt`, `hat` など: `a /op/ \op{a}`

- 一引数一オプションコマンド: `sqrtn`: `a b /op/ \op[b]{a}`

	`7 3 sqrtn` は 7 の 3 乗根 になります。

- 二引数コマンド: `frac`: `a b /op/ \op{a}{b}`

- 単項演算子: `@+`, `@-`, `@+-`, `@-+`: `a /@+/ +a`, `a /@+-/ "\pm a"` など

- 二項演算子: `+`, `-`, `/`, `=` など: `a b /op/ "a op b"`

	`pm` などは `a b /op/ "a \op b"` となります

- 上付き, 下付き: `^`, `_`:  `a b /op/ a^{b}`

	例外: `^` において、`a` が三角関数だった場合、三角関数に上付き文字が付加されます。
	
		a sin	=> \sin a
		2 ^		=> \sin^{2} a


- 括弧類: `[]`, `()`, `{}`, `||`, `<>`: `a /op/ [a]` など

	`<>` の場合は `\langle`, `\rangle` が使われます。

- 括弧類: `\[]`, `\()`, `\{}`, `\||`: `a /op/ \[a\]` など

- 括弧類: `[[]]`, `(())`, `{{}}`: `a /op/ \left [ a \right]` など

  以下の組合せで必ず 4 文字になります。
  `[[))`, `))((` なども可能です。

  `((`, `))`, `{{`, `}}`, `[[`, `]]`, `<<` (\langle), `>>` (\rangle),
  `||`, `\|`, `//`, `\\` (\backslash), `|~` (\lceil), `~|` (\rceil),
  `|_` (\lfloor), `_|` (\rfloor), `..`

- sum 系 (1):

	- `sum`, `int` など: `a /op/ \op a`
    - `sum^`, `int^` など: `b a /op/ \op^{b} a`
    - `sum_`, `int_` など: `b a /op/ \op_{b} a`
    - `sum_^`, `int_^` など: `r1 r2 a /op/ \op_{r1}^{r2} a`
    - `sum^_`, `int^_` など: `r1 r2 a /op/ \op^{r1}_{r2} a`

	operation の対象は常に `op` の直前の要素です。
	`r1 r2 a sum_^` は 「r1 から r2 までの a の和」となります。

- sum 系 (2):
  sum 系のオペレータを大文字で始めた場合は、

  `r1 r2 a /op/ \op_{r1}^{r2} a`

  となります。

  例:

		a r1 r2 Sum => \sum_{r1}^{r2} a

  下付きのみ、上付きのみ、はありません。

  op の後の ^, _ は (現バージョンでは) 許容しますが、無視します。
  つまり `Sum^_` としても `Sum` と扱われ、下付き・上付きの順番は変わり
  ません。
  この仕様は今後変更するかもしれません。

- lim 系:
	- `lim` など: `a /op/ \op a`
    - `lim_` など: `b a /op/ \op_{b} a`

	operation の対象は常に `op` の直前の要素です。

- その他:
    - `a b c /<||>/ \braket{a|b|c}`
    - `a b c /<<||>>/ \Braket{a|b|c}`
    - `a b /<|>/ \braket{a|b}`
    - `a b /<<|>>/ \Braket{a|b}`

  braket.sty が必要です

- ギリシャ文字

  `@a` (`\alpha`), `@G` (`\Gamma`), `@vph` (`\varphi`) など
  `@` の後に `v` を付けると var タイプになります。
  `@A` など対応するコマンドがない (`\Alpha` はない) 場合は、は対応する文字 (`A` など) が入ります。
  `@va` など var タイプが存在しない場合は、入力した文字列そのものがスタックに積まれます。

- 特殊命令

  特殊命令は全て . で始まります (括弧類として .. を使った場合を除く)。

	- `..`, `*` (結合): `a b /op/ "a b"`

	  結果にはスペースが入る点に注意して下さい

	- `...` (スペースなしの結合): `a b /.../ ab`

	- `.exch`, `.swap` (exchange): `a b /.exch/ b a`

	- `.dup` (duplicate): `a /.dup/ a a`

	- `.pop`, `.drop` (drop): `a /.pop/`

	  スタックの最終要素を捨てます。

	- `.rot` (rotate): `a b c /.rot/ b c a`

	- `.over`: `a b /.over/ a b a`

	- `.com`: `arg1 arg2 .. argN com [N] /.com/ \com{arg1}{arg2}...{argN}`
      (default: N = 0)

	  `\com` を N 個の引数を取るコマンドとしてスタックに追加
	  `argX` が [] で囲まれている場合は、そのまま concat します。

	  例:

            newcom .com => \newcom
             arg1  arg2 newcom 2 .com => \newcom{arg1}{arg2}
			[arg1] arg2 newcom 2 .com => \newcom[arg1]{arg2}

	- `.lrparen`: `a LP RP /.lrparen/ \leftLP a \rightRP`

	  現在の仕様では lp, rp には必ず先頭に \ が付きます。
	  (今後変更の場合アリ)。

			a \uparrow .dup .lrparen => \left\uparrow a \right\uparrow


	- `.u`, `.undo` (スクリプトとしての実行時 / Emacsでの実行時)

	  最後の操作 (入力行) を取り消します。
	  モジュールとして使用している場合の動作は未定義です。
	  また、行頭以外での動作も未定義です。

	  例 (スタックが空の状態で以下を実行):

	  (1) スタックは空になります

            a sin b cos
            .undo

	  (2) スタックは `\sin a` になります (`b cos` のみが取り消される)。

	        a sin
            b cos
            .undo

	  (3) 動作は未定義です

	        a sin b sin .undo

	- `.q`, `.quit`, `.exit`

	  終了します (主に通常のスクリプトとして実行した場合に使用します)

	- `.clear`

	  スタックをクリアします

	- `.v`

	  バージョンを表示します

	- `.show`

	  結果を出力します (= スタックの内容を結合して表示します)

	- `."str"`

	  "" 内をそのままスタックに積みます。
	  スペースを含めることもできます (例: `."This is a text."`)。
	  ただし " 自体は str 中に含めることができません。
	  その場合は、`"` と `..` あるいは `...` を組み合わせるなどして下さい。
	  対応する " が無い場合の動作は未定義です。


Acknowlegement
--------------

ギリシャ文字の入力等において、

- [RPN による LaTeX 数式エディタ](http://www.kabipan.com/densan/rpnedit2/rpnedit_jp.html)

を~~パクら~~参考にさせていただきました。
